/**
 * \file
 * My implementation of a ringbuffer.
 * It maps a shared memory object twice directly following
 * thus make it possible to read and write from any
 * position within the buffer without the nasty wrap
 * calculations.
 * This is achived because the same memory region is mapped
 * at the two addresses.
 *
 * \author	Georg Hopp
 *
 * \copyright
 * Copyright © 2014 Georg Hopp
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __TR_CBUF_H__
#define __TR_CBUF_H__

#include <ctype.h>
#include <string.h>
#include <sys/types.h>

#include "trbase.h"
#include "trio.h"

#define TR_ECBUFOVFL		100


TR_CLASS(TR_Cbuf) {
	char * shm_name;  // shared memory identifier

	char * data;
	Bool   lock;

	size_t bsize;
	size_t bused;

	size_t write;
	size_t read;
};
TR_INSTANCE_INIT(TR_Cbuf);
TR_CLASSVARS_DECL(TR_Cbuf) {};

ssize_t TR_cbufRead(TR_Cbuf,  TR_Stream);
ssize_t TR_cbufWrite(TR_Cbuf, TR_Stream);

char *  TR_cbufGetLine(TR_Cbuf, char **);
char *  TR_cbufGetData(TR_Cbuf, size_t);
char *  TR_cbufSetData(TR_Cbuf, const void *, size_t);
void    TR_cbufEmpty(TR_Cbuf);

char *  TR_cbufGetRead(TR_Cbuf this);
char *  TR_cbufGetWrite(TR_Cbuf this);
char *  TR_cbufMemchr(TR_Cbuf this, int c);
size_t  TR_cbufAddrIndex(TR_Cbuf this, const void * c);
void    TR_cbufIncRead(TR_Cbuf this, size_t n);
void    TR_cbufIncWrite(TR_Cbuf this, size_t n);
size_t  TR_cbufGetFree(TR_Cbuf this);
char    TR_cbufIsEmpty(TR_Cbuf this);
void    TR_cbufSkipNonAlpha(TR_Cbuf this);
Bool    TR_cbufIsLocked(TR_Cbuf this);
void    TR_cbufLock(TR_Cbuf this);
void    TR_cbufRelease(TR_Cbuf this);

#endif // __TR_CBUF_H__

// vim: set ts=4 sw=4:
