/**
 * \file
 *
 * \author	Georg Hopp
 *
 * \copyright
 * Copyright © 2014 Georg Hopp
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __TR_TREE_H__
#define __TR_TREE_H__

#include "tr/tree_macros.h"
#include "trbase.h"

TR_CLASS(TR_TreeNode) {
	void * data;

	TR_rbColor color;

	TR_TreeNode parent;
	TR_TreeNode left;
	TR_TreeNode right;
};
TR_INSTANCE_INIT(TR_TreeNode);
TR_CLASSVARS_DECL(TR_TreeNode) {};

TR_CLASS(TR_Tree) {
	TR_TreeNode     root;
};
TR_INSTANCE_INIT(TR_Tree);
TR_CLASSVARS_DECL(TR_Tree) {};

typedef int  (*TR_TreeComp)(const void *, const void *);
typedef void (*TR_TreeAction)(const void *, const void *, const int);

void          * TR_treeFind(TR_Tree, const void *, TR_TreeComp);
void          * TR_treeInsert(TR_Tree, const void *, TR_TreeComp);
void          * TR_treeDelete(TR_Tree, const void *, TR_TreeComp);
unsigned long   TR_treeWalk(TR_Tree, const void *, TR_TreeAction);
void            TR_treeDestroy(TR_Tree, TR_TreeAction);

#endif // __TR_TREE_H__

// vim: set ts=4 sw=4:
