/**
 * \file
 *
 * \author	Georg Hopp
 *
 * \copyright
 * Copyright © 2014 Georg Hopp
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stdarg.h>

#include "trbase.h"
#include "tr/dynarray.h"

static
inline
void
_darrResize(TR_Dynarray this, size_t index)
{
	if (!this->size || index > this->size - 1) { 
		void * new_data = TR_calloc(index + 1, sizeof(void *));
		memcpy(new_data, this->data, this->size * sizeof(void *));
		TR_MEM_FREE(this->data);
		this->data = (const void **)new_data;
		this->size = TR_getUsableSize(new_data) / sizeof(void *);
	}
}

size_t
TR_darrPut(TR_Dynarray this, const void * data)
{
	size_t i = TR_darrFindFirstFree(this);

	TR_darrPutAt(this, data, i);

	return i;
}

void
TR_darrPutAt(TR_Dynarray this, const void * data, size_t idx)
{
	_darrResize(this, idx);
	this->data[idx] = data;
}

// vim: set ts=4 sw=4:
