/**
 * \file
 *
 * \author	Georg Hopp
 *
 * \copyright
 * Copyright © 2014 Georg Hopp
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stdarg.h>
#include <sys/types.h>

#include "trbase.h"
#include "tr/heap.h"
#include "tr/dynarray.h"

const void *
TR_heapGet(TR_Heap this)
{
	const void * value = TR_darrGet((TR_Dynarray)this, 0);
	size_t       left  = 1;
	size_t       right = 2;
	size_t       idx;

	if (! value) {
		return NULL;
	}

	idx = TR_darrFindFirstFree((TR_Dynarray)this) - 1;
	TR_darrGet((TR_Dynarray)this, 0) = NULL;

	if (0 == idx) {
		return value;
	}

	const void * tmp = TR_darrGet((TR_Dynarray)this, 0);
	TR_darrPutAt((TR_Dynarray)this, TR_darrGet((TR_Dynarray)this, idx), 0);
	TR_darrPutAt((TR_Dynarray)this, tmp, idx);
	idx = 0;

	while (left < ((TR_Dynarray)this)->size &&
			TR_darrGet((TR_Dynarray)this, left)) {
		size_t change = left;

		if (right < ((TR_Dynarray)this)->size &&
				TR_darrGet((TR_Dynarray)this, right) &&
				0 > this->comp (
					TR_darrGet((TR_Dynarray)this, left),
					TR_darrGet((TR_Dynarray)this, right))) {
			change = right;
		}

		if (0 > this->comp(
					TR_darrGet((TR_Dynarray)this, idx),
					TR_darrGet((TR_Dynarray)this, change))) {
			tmp = TR_darrGet((TR_Dynarray)this, change);
			TR_darrPutAt(
					(TR_Dynarray)this,
					TR_darrGet((TR_Dynarray)this, idx),
					change);
			TR_darrPutAt((TR_Dynarray)this, tmp, idx);
			idx   = change;
			left  = (idx << 1) + 1;
			right = left + 1;
		} else {
			break;
		}
	}

	return value;
}

// vim: set ts=4 sw=4:
