/**
 * \file
 *
 * \author	Georg Hopp
 *
 * \copyright
 * Copyright © 2014 Georg Hopp
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stdarg.h>
#include <sys/types.h>

#include "trbase.h"
#include "tr/heap.h"
#include "tr/dynarray.h"

void
TR_heapPut(TR_Heap this, const void * data)
{
	size_t idx = TR_darrPut((TR_Dynarray)this, data);

	while (idx) {
		size_t parent = (idx - 1) >> 1;

		if (0 > this->comp(
				TR_darrGet((TR_Dynarray)this, parent),
				TR_darrGet((TR_Dynarray)this, idx))) {
			const void * tmp = TR_darrGet((TR_Dynarray)this, parent);
			TR_darrPutAt(
					(TR_Dynarray)this,
					TR_darrGet((TR_Dynarray)this, idx),
					parent);
			TR_darrPutAt((TR_Dynarray)this, tmp, idx);
			idx = parent;
		} else {
			break;
		}
	}
}

// vim: set ts=4 sw=4:
