/**
 * \file
 *
 * \author	Georg Hopp
 *
 * \copyright
 * Copyright © 2014 Georg Hopp
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stdarg.h>

#include "trbase.h"
#include "tr/list.h"
#include "tr/interface/iterable.h"

static
int
listCtor(void * _this, va_list * params)
{
	TR_List this = _this;

	this->data  = (void **)TR_malloc(32 * sizeof(void *));
	this->size  = TR_getUsableSize(this->data) / sizeof(void *);
	this->start = this->end = this->current = 0;
	this->free_msgs = 1;

	return 0;
}

static
void
listDtor(void * _this)
{
	TR_List this = _this;
	size_t  i;

	if (this->free_msgs) {
		for (
				i = this->start;
				i != this->end;
				i = i + 1 == this->size ? 0 : i + 1) {
			if (this->data[i]) {
				TR_delete(this->data[i]);
			}
		}
	}

	TR_MEM_FREE(this->data);
}

static
void *
listCurrent(void * _this)
{
	TR_List this = _this;
	return this->data[this->current];
}

static
void
listNext(void * _this)
{
	TR_List this = _this;
	this->current = this->current + 1 == this->size ? 0 : this->current + 1;
}

static
void
listRewind(void * _this)
{
	TR_List this = _this;
	this->current = this->start;
}

static
int
listValid(void * _this)
{
	TR_List this = _this;
	return this->current != this->end;
}

TR_INIT_IFACE(TR_Class, listCtor, listDtor, NULL);
TR_INIT_IFACE(TR_Iterable, listCurrent, listNext, listRewind, listValid);
TR_CREATE_CLASS(TR_List, NULL, NULL, TR_IF(TR_Class), TR_IF(TR_Iterable));

// vim: set ts=4 sw=4:
