/**
 * \file
 *
 * \author	Georg Hopp
 *
 * \copyright
 * Copyright © 2014 Georg Hopp
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stdarg.h>

#include "trbase.h"
#include "tr/set.h"
#include "tr/interface/iterable.h"

static
int
setCtor(void * _this, va_list * params)
{
	TR_Set this = _this;

	this->data  = (void **)TR_calloc(32, sizeof(void *));
	this->size  = TR_getUsableSize(this->data) / sizeof(void *);
	this->free_msgs = 1;

	return 0;
}

static
void
setDtor(void * _this)
{
	TR_Set this = _this;
	size_t  i;

	if (this->free_msgs) {
		TR_iterableForeach(this) {
			TR_delete(this->data[i]);
		}
	}

	TR_MEM_FREE(this->data);
}

static
void *
setCurrent(void * _this)
{
	TR_Set this = _this;
	return this->data[this->current];
}

static
void
setNext(void * _this)
{
	TR_Set this = _this;
	++this->current;
}

static
void
setRewind(void * _this)
{
	TR_Set this = _this;
	this->current = 0;
}

static
int
setValid(void * _this)
{
	TR_Set this = _this;
	return this->current < this->used;
}

TR_INIT_IFACE(TR_Class, setCtor, setDtor, NULL);
TR_INIT_IFACE(TR_Iterable, setCurrent, setNext, setRewind, setValid);
TR_CREATE_CLASS(TR_Set, NULL, NULL, TR_IF(TR_Class), TR_IF(TR_Iterable));

// vim: set ts=4 sw=4:
