/**
 * \file
 *
 * \author	Georg Hopp
 *
 * \copyright
 * Copyright © 2014 Georg Hopp
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "trbase.h"
#include "tr/tree.h"

void *
TR_treeInsert(TR_Tree this, const void * search, TR_TreeComp comp)
{
	TR_TreeNode   new_node = NULL;
	TR_TreeNode   node;
	int           found;
	void        * retval;

	node = this->root;

	/*
	 * insert the node or return the one in tree if comparison
	 * succeeds.
	 */
	if (NULL == node) {
		/*
		 * if the root is NULL we simple add the element and set
		 * node to it.
		 */
		this->root = node = new_node = TR_new(TR_TreeNode, search);
	} else {
		TR_TREE_FIND(node, search, found, comp);

		if (found == 0) {
			// This differs from tsearch, which is the posix equivalent to
			// this function in that it will not replace an existing value.
			// If there is a value for the given key in the tree it will be
			// retured. It is then up to the caller to handle the situation.
			// This is used in my http header code to handle multiple value
			// per header. There it is more useful to get the already existing
			// tree entry and add the new data.
			return node->data;
		} else {
			// not found
			if (0 < found) {
				node->left = TR_new(TR_TreeNode, search);
				node->left->parent = node;
				node = new_node = node->left;
			} else {
				node->right = TR_new(TR_TreeNode, search);
				node->right->parent = node;
				node = new_node = node->right;
			}
		}
	}

	/*
	 * we expect node not to be NULL and pointing to our
	 * new node at this point...now rabalance the tree
	 */
	TR_TREE_BALANCE_INSERT(&(this->root), node);
	retval = new_node->data;

	return retval;
}

// vim: set ts=4 sw=4:
