/**
 * \file
 *
 * \author	Georg Hopp
 *
 * \copyright
 * Copyright © 2014 Georg Hopp
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "tr/tree.h"

unsigned long
TR_treeWalk(TR_Tree this, const void * data, TR_TreeAction action)
{
	TR_TreeNode   previous;
	TR_TreeNode   node;
	int           depth     = 1;
	unsigned long processed = 0;

	previous = this->root;
	node     = this->root;

	while (NULL != node) {
		if (previous == TR_TREE_RIGHT(node)) {
			previous = node;
			node     = node->parent;
			depth--;
			continue;
		}

		if (NULL == TR_TREE_LEFT(node) || previous == TR_TREE_LEFT(node)) {
			if (action) {
				action(node->data, data, depth);
				processed++;
			}
			previous = node;

			if (NULL != TR_TREE_RIGHT(node)) {
				node = TR_TREE_RIGHT(node);
				depth++;
			} else {
				node = TR_TREE_PARENT(node);
				depth--;
			}
		} else {
			previous = node;
			node     = TR_TREE_LEFT(node);
			depth++;
		}
	}

	return processed;
}

// vim: set ts=4 sw=4:
